﻿using ProjectBoat.CollectionGenericObjects;
using ProjectBoat.Drawnings;
using System.Windows.Forms;

namespace ProjectBoat
{
    public partial class FormBoatsCollection : Form
    {
        /// <summary>
        /// Хранилише коллекций
        /// </summary>
        private readonly StorageCollection<DrawningBoat> _storageCollection;

        /// <summary>
        /// Компания
        /// </summary>
        private AbstractCompany? _company = null;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBoatsCollection()
        {
            InitializeComponent();
            _storageCollection = new();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBoxSelectorCompany_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }

        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {
            FormBoatConfing form = new();
            // TODO передать метод

            form.Show();
            form.AddEvent(SetBoat);
        }

        /// <summary>
        /// Добавление лодки в коллекцию
        /// </summary>
        /// <param name="boat"></param>
        private void SetBoat(DrawningBoat boat)
        {
            if (_company == null || boat == null)
            {
                return;
            }

            if (_company + boat != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBoxBoat.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }

        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosision.Text) || _company == null)
            {
                return;
            }
            if (MessageBox.Show("удалить объект?", "удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosision.Text);
            if (_company - pos != null)
            {
                MessageBox.Show("объект удален");
                pictureBoxBoat.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("не удалось удалить объект");
            }
        }

        private void ButtonGetToTest_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            DrawningBoat? boat = null;
            int counter = 100;
            while (boat == null)
            {
                boat = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
            if (boat == null)
            {
                return;
            }
            FormBoat form = new()
            {
                SetBoat = boat
            };
            form.ShowDialog();

        }

        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            pictureBoxBoat.Image = _company.Show();
        }

        /// <summary>
        /// Добавление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollecctionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }
            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Удаленние коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollectionDel_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }
            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Обновление списка в listBoxCollection
        /// </summary>
        private void RerfreshListBoxItems()
        {
            listBoxCollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i];
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxCollection.Items.Add(colName);
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateCompany_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }
            ICollectionGenericObjects<DrawningBoat>? collection =
                _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не проинициализирована");
                return;
            }
            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new BoatDockingService(pictureBoxBoat.Width, pictureBoxBoat.Height, collection);
                    break;
            }
            panelCompanyTools.Enabled = true;

        }


    }
}
